import gtk
import gobject
import cPickle
from os import sep
from xpn_src.UserDir import userdir

class Custom_List(gtk.GenericTreeModel):
    def __init__(self,list=[]):
        gtk.GenericTreeModel.__init__(self)
        self.list=list
    
    def on_get_flags(self):
        return gtk.TREE_MODEL_LIST_ONLY|gtk.TREE_MODEL_ITERS_PERSIST
        
    def on_get_n_columns(self):
        return 2
    
    def on_get_column_type(self, index):
        return gobject.TYPE_STRING
        
    def on_get_iter(self, path):
        return path[0]
        
    def on_get_path(self, rowref):
        return tuple(rowref)
        
    def on_get_value(self, rowref, column):
        try: 
            return self.list[rowref][column]
        except IndexError:
            return None
        
    def on_iter_next(self, rowref):
        try:
            self.list[rowref+1]
        except IndexError:
            return None
        else:
            return rowref+1

    def on_iter_children(self, parent):
        return None

    def on_iter_has_child(self, rowref):
        return False
        
    def on_iter_n_children(self, rowref):
        if rowref:
            return 0
        return len(self.list)
        
    def on_iter_nth_child(self, parent, n):
        if parent:
            return None
        try:
            self.list[n]
        except IndexError:
            return None
        else:
            return n
        
    def on_iter_parent(self, child):
        return None

class Groups_List:
    def get_widget(self):
        return self.scrolledwin

    def show(self):
        self.scrolledwin.show_all()

    def hide(self):
        self.scrolledwin.hide_all()

    def clear(self):
        self.model=Custom_List()
        self.groups_list.set_model(self.model)

    def show_list(self,list):
        new_model=Custom_List(list)
        self.groups_list.set_model(new_model)
        self.model=new_model
        
    def __init__(self,column1_name,column2_name):
        #GroupsScrolledWin
        self.scrolledwin=gtk.ScrolledWindow()
        self.scrolledwin.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
        self.scrolledwin.set_shadow_type(gtk.SHADOW_OUT)

        self.model=Custom_List()
        try:
            f=open(userdir + sep +"dats/sizes.dat","rb")
        except IOError:
            column1_width=145
        else:
            sizes=cPickle.load(f)
            f.close()
            column1_width=int(sizes["groups_col1"])

        #GroupsTree
        self.groups_list=gtk.TreeView(self.model)
        text_renderer=gtk.CellRendererText()
        self.column1=gtk.TreeViewColumn(column1_name,text_renderer,text=0)
        self.column2=gtk.TreeViewColumn(column2_name,text_renderer,text=1)
        self.column1.set_resizable(gtk.TRUE)
        self.column1.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        self.column1.set_fixed_width(column1_width)
        self.column2.set_resizable(gtk.FALSE)
        self.column2.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        self.groups_list.append_column(self.column1)
        self.groups_list.append_column(self.column2)
        self.scrolledwin.add(self.groups_list)
        self.groups_list.set_property("fixed-height-mode",gtk.TRUE)


class Groups_Pane:
    def get_widget(self):
        return self.scrolledwin

    def unparent(self):
        self.scrolledwin.unparent()

    def show(self):
        self.scrolledwin.show_all()

    def hide(self):
        self.scrolledwin.hide_all()

    def clear(self):
        self.model.clear()

    def set_background(self,color):
        color=gtk.gdk.color_parse(color)
        self.groups_list.modify_base(gtk.STATE_NORMAL,color)

    def set_foreground(self,color):
        color=gtk.gdk.color_parse(color)
        self.groups_list.modify_text(gtk.STATE_NORMAL,color)
        self.groups_list.modify_fg(gtk.STATE_NORMAL,color)


    def append(self,values):
        iter=self.model.append(values)
        return iter

    def show_list(self,list,apply_bold=False):
        self.clear()
        if apply_bold==True:
            for group in list:
                if int(group[1].split(" ")[0])>0:
                    self.append((group[0],group[1],gtk.TRUE))
                else:
                    self.append((group[0],group[1],gtk.FALSE))
        else:
            for group in list:
                self.append((group[0],group[1],gtk.FALSE))

    def view_next_group(self,obj):
        treesel=self.groups_list.get_selection()
        model,iter=treesel.get_selected()
        column=self.groups_list.get_column(0)
        self.groups_list.grab_focus()
        if iter==None:
            path=(0,)
        else:
            path=model.get_path(iter)
            path=(path[0]+1,)
        self.groups_list.set_cursor(path,None,gtk.FALSE)
        self.groups_list.row_activated(path,column)


    def __init__(self,column1_name,column2_name,enable_weight,configs):
        #GroupsScrolledWin
        self.scrolledwin=gtk.ScrolledWindow()
        self.scrolledwin.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
        self.scrolledwin.set_shadow_type(gtk.SHADOW_OUT)

        #Model
        self.model=gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_BOOLEAN)

        try:
            f=open(userdir + sep +"dats/sizes.dat","rb")
        except IOError:
            column1_width=145
        else:
            sizes=cPickle.load(f)
            f.close()
            column1_width=int(sizes["groups_col1"])

        #GroupsTree
        self.groups_list=gtk.TreeView(self.model)
        text_renderer=gtk.CellRendererText()
        if enable_weight:
            text_renderer.set_property("weight",1000)
        self.column1=gtk.TreeViewColumn(column1_name,text_renderer,text=0,weight_set=2)
        self.column2=gtk.TreeViewColumn(column2_name,text_renderer,text=1,weight_set=2)
        self.column1.set_resizable(gtk.TRUE)
        self.column1.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        self.column1.set_fixed_width(column1_width)
        self.column1.set_sort_column_id(0)
        self.column2.set_resizable(gtk.FALSE)
        self.column2.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        self.groups_list.append_column(self.column1)
        self.groups_list.append_column(self.column2)
        self.scrolledwin.add(self.groups_list)

        if enable_weight:
            color=configs["background_color"]
            self.set_background(color)
            color=configs["text_color"]
            self.set_foreground(color)
        #self.groups_list.set_property("fixed-height-mode",gtk.TRUE)

