import cPickle
import shelve
from nntplib import *
import socket
import time
import gtk
import sys
from xpn_src.Article import Article
from xpn_src.Score import Score_Rules
from os import remove, sep
from xpn_src.UserDir import userdir

class ExportNewsrc:
    def open_groups_list(self):
        "returns the newsgroups list"
        try:
            f=open(userdir + sep +"groups_info/groups.dat","rb")
        except:
            return None
        else:
            groups_list=cPickle.load(f)
            f.close()
            return groups_list

    def open_subscribed_list(self):
        "returns subscribed_list and the read_articles lists"
        try:
            f=open(userdir + sep +"groups_info/subscribed.dat","rb")
        except IOError:
            return None,None
        else:
            list=cPickle.load(f)
            f.close()
            name_list=[]
            read_list={}
            for group in list:
                try:
                    f2=open(userdir + sep +"groups_info/"+group[0],"rb")
                except IOError:
                    return list,None
                else:
                    f2.close()
                    name_list.append(group[0])
                    last=group[1]
                    articles=shelve.open(userdir + sep +"groups_info/"+group[0])
                    read_articles=range(1,int(last)+1)
                    for article in articles.itervalues():
                        if not article.is_read:
                            #the article is unread
                            read_articles.remove(int(article.number))
                    articles.close()
                    read_list[group[0]]=read_articles
            return name_list,read_list

    def compact_list(self,L):
        intervals = [[L[0]] * 2]
        for item in L:
            if item == 1+intervals[-1][-1]:
                intervals[-1][-1] = item
            else:
                intervals.append([item]*2)
        intervals.pop(0)
        return intervals

    def format_intervals(self,intervals):
        string=""
        for interval in intervals:
            if interval[0]==interval[1]:
                string=string+str(interval[0])+","
            else:
                string=string+str(interval[0])+"-"+str(interval[1])+","
        return string[0:-1]

    def build_line(self,group,subscribed_list,read_list):
        if subscribed_list!=None:
            if group in subscribed_list:
                if read_list[group]!=None and read_list[group]!=[]:
                    clist=self.format_intervals(self.compact_list(read_list[group]))
                else:
                    clist=""
                return (group+": "+clist).strip()
            else:
                return group+"!"
        else:
            return group+"!"

    def build_newsrc(self):
        subscribed_list,read_list=self.open_subscribed_list()
        groups_list=self.open_groups_list()
        if groups_list==None:
            return None
        else:
            newsrc=[]
            for group in groups_list:
                line=self.build_line(group[0],subscribed_list,read_list)+"\n"
                newsrc.append(line)
            newsrc="".join(newsrc)
            return newsrc

    def save_newsrc(self,where):
        try:
            f=open(where,"wb")
        except IOError:
            pass
        else:
            if self.newsrc!=None:
                f.write(self.newsrc)
            f.close()


    def __init__(self):
        self.message=""
        self.newsrc=""
        self.newsrc=self.build_newsrc()
        if self.newsrc==None:
            self.message="You have to download the newsgroups list first"
        else:
            self.message="Newsrc successful exported"



class ImportNewsrc:
    def try_connection(self):
        if self.server==None:
            try:
                if self.configs["username"]!="":
                    self.add_log("AUTHINFO USER "+self.configs["username"],True)
                    self.add_log("AUTHINFO PASS "+self.configs["password"],True)
                self.add_log("MODE READER",True)
                
                self.server=NNTP(self.configs["server"],port=int(self.configs["port"]),user=self.configs["username"],password=self.configs["password"],readermode=True)
            except:
                message="No connection with server : "+self.configs["server"]+". Configure NNTP Server Address or try later."
                self.add_log(str(sys.exc_info()[0])+","+str(sys.exc_info()[1]),False)
            else:
                message="Connection estabilished with:"+self.configs["server"]
                self.add_log(self.server.getwelcome(),False)
            self.main_win.statusbar.push(1,message)

    def add_log(self,message,is_command):
        try:
            f=open(userdir + sep +"server_logs.dat","a")
        except IOError:
            pass
        else:
            if is_command:
                f.write(time.ctime(time.time())+" :: >> "+message+"\n")
            else:
                f.write(time.ctime(time.time())+" :: << "+message+"\n")
            f.close()

    def extract_ranges(self,L):
        list=[]
        if L!="":
            intervals=L.split(",")
            for interval in intervals:
                if "-" in interval:
                    #this is a range
                    start,stop=int(interval.split("-")[0]),int(interval.split("-")[1])
                    list.extend(range(start,stop+1))
                else:
                    #this is a single number
                    list.append(int(interval))
        return list

    def find_first_unread(self,L):
        first_interval=L.split(",")[0]
        if "-" in first_interval:
            if first_interval.split("-")[0]=="1":
                first_unread=int(first_interval.split("-")[1])+1
            else:
                first_unread=1
        else:
            if first_interval=="1":
                first_unread=2
            else:
                first_unread=1
        return first_unread

    def subscribe_groups(self,newsrc):
        for group in newsrc:
            if group.find(":")!=-1:
                #this is a subscribed group
                group_info=group.split(":")
                group_name=group_info[0].strip()
                group_read_ranges=group_info[1].strip()
                group_read_list=self.extract_ranges(group_read_ranges)
                group_first_unread=self.find_first_unread(group_read_ranges)
                self.subscribe_group(group_name,group_first_unread,group_read_list)

    def retrieve_body(self,article_to_read):
        body=article_to_read.get_body()
        if body==None:
            self.try_connection()
            body=[]
            if self.server!=None:
                try:
                    self.add_log("HEAD "+article_to_read.number,True)
                    resp,number,msgid,header_list=self.server.head(article_to_read.number)
                except NNTPTemporaryError:
                    if str(sys.exc_info()[1])[:1]=="4":
                        #article is not on the server, we use a phantom article
                        link=r"http://groups.google.com/groups?ie=UTF-8&oe=UTF-8&as_umsgid="+url_quote(article_to_read.msgid[1:-1])
                        body=("Server Error: "+str(sys.exc_info()[1]),"","You can try on Google:","",link)
                    else:
                        self.statusbar.push(1,"Server Error: str(sys.exc_info()[1])")
                        #VALUTARE SE BISOGNA GESTIRE LA SITUAZIONE!!!
                else:
                    self.add_log(resp,False)
                    article_to_read.parse_header_list(header_list)
                    self.add_log("BODY "+article_to_read.number,True)
                    resp,number,id,list=self.server.body(article_to_read.number)
                    self.add_log(resp,False)
                    article_to_read.set_body(list)
                    body=article_to_read.get_body()
        return body


    def subscribe_group(self,group_to_subscribe,first_unread,read_list):
        if self.server!=None:
            try:
                self.add_log("GROUP "+group_to_subscribe,True)
                resp,count,first,last,name=self.server.group(group_to_subscribe)
            except:
                self.main_win.statusbar.push(1,"Server error: "+str(sys.exc_info()[1]))
                self.add_log(str(sys.exc_info()[0])+","+str(sys.exc_info()[1]),False)
                self.add_log("QUIT",True)
                self.add_log(self.server.quit(),False)
            else:
                watched=self.get_watched()
                ignored=self.get_ignored()
                subscribed_groups=[]
                self.add_log(resp,False)
                try:
                    f=open(userdir + sep +"groups_info/subscribed.dat","rb")
                except IOError:
                    subscribed_groups.append([group_to_subscribe,last])
                else:
                    subscribed_groups=cPickle.load(f)
                    f.close()
                    if subscribed_groups:
                        already_in=False
                        for group in subscribed_groups:
                            if group[0]==group_to_subscribe:
                                already_in=True
                                to_remove=group
                        if already_in:
                            subscribed_groups.remove(to_remove) 
                    subscribed_groups.append([group_to_subscribe,last])

                    #refresh the subscribed file
                try:
                    f=open(userdir + sep +"groups_info/subscribed.dat","wb")
                except IOError:
                    self.main_win.statusbar.push(1,"Problems while opening subscribed.dat")
                else:
                    subscribed_groups.sort()
                    cPickle.dump(subscribed_groups,f,1)
                    f.close()
                    score_rules=Score_Rules()

                    articles=shelve.open(userdir + sep +"groups_info/"+group_to_subscribe)
                    articles.clear()


                    if first_unread<int(first):
                        first_unread=int(first)

                    if first_unread<=int(last):
                        #dowload the headers for the subscribed group

                        first=str(first_unread)

                        self.main_win.progressbar.set_text("Fetching Headers")
                        self.main_win.progressbar.set_fraction(1/float(2))
                        while gtk.events_pending():
                            gtk.main_iteration(gtk.FALSE)
                        self.add_log("XOVER "+first+"-"+last,True)
                        resp,total_headers=self.server.my_xover(first,last)
                        self.add_log(resp,False)
                        self.main_win.statusbar.push(1,self.configs["server"]+" response : "+resp)

                        self.main_win.progressbar.set_text("Building Articles")
                        self.main_win.progressbar.set_fraction(2/float(2))
                        while gtk.events_pending():
                            gtk.main_iteration(gtk.FALSE)
                        for headers in total_headers:
                            number,subject,from_name,date,msgid,references,bytes,lines,xref=headers
                            xpn_article=Article(number,msgid,from_name,references,subject,date,self.configs["fallback_charset"],group_to_subscribe)
                            try:
                                index=references.rindex("<")
                            except ValueError:
                                last_ref=""
                            else:
                                last_ref=references[index:]
                            #applying score rules
                            score=score_rules.apply_score_rules(xpn_article,group_to_subscribe,xref,bytes,lines)
                            xpn_article.set_score(score)
                            #applying score actions
                            xpn_article,actions,watched,ignored=score_rules.apply_action_rules(xpn_article,group_to_subscribe,xref,bytes,lines,watched,ignored)
                            if not "kill" in actions:
                                if ((self.configs["download_bodies"]=="True" and not "markread" in actions) or ("retrieve" in actions) or (last_ref in watched)) and not last_ref in ignored:
                                    body=self.retrieve_body(xpn_article)
                                if last_ref in ignored:
                                    xpn_article.is_read=True
                                    xpn_article.ignore=True
                                    ignored.add(xpn_article.msgid)
                                    watched.discard(xpn_article.msgid)
                                    xpn_article.watch=False
                                if last_ref in watched:
                                    xpn_article.watch=True
                                    watched.add(xpn_article.msgid)
                                    ignored.discard(xpn_article.msgid)
                                    xpn_article.ignore=False
      
                                if int(xpn_article.number) in read_list:
                                    xpn_article.is_read=True

                                articles[xpn_article.msgid]=xpn_article


                        self.main_win.statusbar.push(1,"Group subscribed")
                        self.main_win.progressbar.set_fraction(0)
                        self.main_win.progressbar.set_text("")
                    articles.close()
                self.dump_watched(watched)
                self.dump_ignored(ignored)

    def delete_files(self):
        try:
            f=open(userdir + sep +"groups_info/subscribed.dat","rb")
        except IOError:
            pass
        else:
            subscribed_list=cPickle.load(f)
            for group in subscribed_list:
                try:
                    remove(userdir + sep +"groups_info/"+group[0])
                except:
                    pass
            try:
                remove(userdir + sep +"groups_info/subscribed.dat")
            except:
                pass
    def get_watched(self):
        try:
            f=open(userdir + sep +"dats/watch.dat","rb")
        except IOError:
            watched=set()
        else:
            watched=cPickle.load(f)
            f.close()
        return watched

    def dump_watched(self,watched):
        f=open(userdir + sep +"dats/watch.dat","wb")
        cPickle.dump(watched,f,1)
        f.close()

    def get_ignored(self):
        try:
            f=open(userdir + sep +"dats/ignore.dat","rb")
        except IOError:
            ignored=set()
        else:
            ignored=cPickle.load(f)
            f.close()
        return ignored

    def dump_ignored(self,ignored):
        f=open(userdir + sep +"dats/ignore.dat","wb")
        cPickle.dump(ignored,f,1)
        f.close()

    def __init__(self,newsrc,main_win,configs):
        self.main_win=main_win
        self.configs=configs
        self.server_name=self.configs["server"]

        #cleaning files
        self.delete_files()

        #opening connection with server
        self.server=None
        self.try_connection()

        #subscribing groups
        self.subscribe_groups(newsrc)

        #closing connection with server
        if self.server!=None:
            self.add_log("QUIT",True)
            self.add_log(self.server.quit(),False)
